// ALU Module
`include "instruction.v"
//4bit Adder
module mAdder4(dat1,dat2,cin,out,cout);
	input [3:0] dat1;
	input [3:0] dat2;
	input cin;
	output [3:0] out;
	output cout;
	wire [4:0] wout;
	
	assign wout=dat1+dat2+cin;
	assign cout=wout[4];
	assign out=wout[3:0];
endmodule

//8bit Adder
module mAdder8(dat1,dat2,cin,out,cout,dc);
	input [7:0] dat1;
	input [7:0] dat2;
	input cin;
	output [7:0] out;
	output cout,dc;
	wire wdc;
	
	assign dc=wdc;
	mAdder4 add4a(dat1[3:0],dat2[3:0],cin,out[3:0],wdc);
	mAdder4 add4b(dat1[7:4],dat2[7:4],wdc,out[7:4],cout);
endmodule

//BitMask
module mBitMask(bitin,mask);
	input [2:0] bitin;
	output [7:0] mask;
	reg [7:0] maskdat;
	assign mask=maskdat;
	always @(bitin) begin
		case (bitin)
			3'd0: maskdat=8'h01;
			3'd1: maskdat=8'h02;
			3'd2: maskdat=8'h04;
			3'd3: maskdat=8'h08;
			3'd4: maskdat=8'h10;
			3'd5: maskdat=8'h20;
			3'd6: maskdat=8'h40;
			3'd7: maskdat=8'h80;
		endcase
	end
endmodule

//WReg
module mWreg(reset,clk,Q,write,datin,datout);
	input reset,clk;
	input Q;
	input write;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] wreg;
	assign datout=wreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			wreg<=8'h00;
		end else if(Q) begin
			if(write) begin
				wreg<=datin;
			end
		end
	end
endmodule


		
//ALU Module
module mALU(inst,cmd,wreg,datin,cin,datout,cout,dc,zf,wr_f,wr_w,wr_c,wr_dc,wr_zf,cmd_skip);
	input [5:0] inst;
	input [13:0] cmd;
	input [7:0] wreg;
	input [7:0] datin;
	input cin;
	output [7:0] datout;
	output cout,dc,zf,wr_f,wr_w,wr_c,wr_dc,wr_zf,cmd_skip;
	wire [7:0] addout;
	wire [7:0] w_wreg;
	wire [7:0] w_datin;
	wire [7:0] w_datout;
	wire [7:0] bitmask;
	wire acin,acout;

	//Adder(ADD/SUB)
	assign acin=(inst==`SUBWF) ? 1'b1 : 1'b0;
	assign w_wreg=((inst==`SUBWF)||(inst==`SUBLW)) ? ~wreg :
		((inst==`INCF)||(inst==`INCFSZ)) ? 8'h01 :
		((inst==`DECF)||(inst==`DECFSZ)) ? 8'hff :
		((inst==`BSF)||(inst==`BTFSC)||(inst==`BTFSS)) ? bitmask :
		(inst==`BCF) ? ~bitmask :	wreg;
	assign w_datin=((inst==`ADDLW)||(inst==`SUBLW)||(inst==`ANDLW)||(inst==`IORLW)||(inst==`MOVLW)||(inst==`RETLW)||(inst==`XORLW)) ? cmd[7:0] : datin;
	//Adder
	mAdder8 adder(w_wreg,w_datin,acin,addout,acout,dc);
	//bitmask
	mBitMask bmask(cmd[9:7],bitmask);
	
	assign cout=(inst==`RLF) ? datin[7] :
		(inst==`RRF) ? datin[0] : acout;
	assign datout=w_datout;
	assign zf=(w_datout==8'h00) ? 1'b1 : 1'b0;
	
	//ALU Main
	assign w_datout=((inst==`ADDLW)||(inst==`ADDWF)||(inst==`SUBLW)||(inst==`SUBWF)||(inst==`INCF)||(inst==`INCFSZ)||(inst==`DECF)||(inst==`DECFSZ)) ? addout  :	//Addtion Function
			((inst==`ANDWF)||(inst==`ANDLW)||(inst==`BCF)||(inst==`BTFSC)||(inst==`BTFSS)) ? (w_wreg & w_datin) :				//AND
			((inst==`CLRF)||(inst==`CLRW)) ? 8'h00  :		//CLR
			((inst==`IORWF)||(inst==`BSF)||(inst==`IORLW))	? (w_datin | w_wreg) :	//OR
			(inst==`RLF) ? {w_datin[6:0],cin} :		//SHIFT LEFT
			(inst==`RRF) ? {cin,w_datin[7:1]} :		//SHIFT RIGHT
			((inst==`XORWF)||(inst==`XORLW)) ? (w_wreg ^ w_datin) :	//XOR
			(inst==`SWAPF) ? {w_datin[3:0],w_datin[7:4]} : //Swap
			(inst==`COMF) ? ~w_datin :	//Complement
			((inst==`MOVF)||(inst==`MOVLW)||(inst==`RETLW)) ? w_datin : w_wreg;	//File Reg :othere=wreg
	//assign ,
	assign wr_f=((((inst==`ADDWF)||(inst==`ANDWF)||(inst==`CLRF)||(inst==`COMF)||(inst==`DECF)||(inst==`DECFSZ)||(inst==`INCF)||(inst==`INCFSZ)||
			(inst==`IORWF)||(inst==`MOVF)||(inst==`RLF)||(inst==`RRF)||(inst==`SUBWF)||(inst==`SWAPF)||(inst==`XORWF)) && cmd[7])||
			((inst==`BCF)||(inst==`BSF)||(inst==`MOVWF))) ? 1'b1 : 1'b0;
	assign wr_w=((((inst==`ADDWF)||(inst==`ANDWF)||(inst==`CLRW)||(inst==`COMF)||(inst==`DECF)||(inst==`DECFSZ)||(inst==`INCF)||(inst==`INCFSZ)||
			(inst==`IORWF)||(inst==`MOVF)||(inst==`RLF)||(inst==`RRF)||(inst==`SUBWF)||(inst==`SWAPF)||(inst==`XORWF)) && ~cmd[7])||
			((inst==`ANDLW)||(inst==`IORLW)||(inst==`MOVLW)||(inst==`RETLW)||(inst==`XORLW))) ? 1'b1 : 1'b0;
	assign wr_c=((inst==`ADDWF)||(inst==`RLF)||(inst==`RRF)||(inst==`SUBWF)) ? 1'b1 : 1'b0;
	assign wr_dc=((inst==`ADDWF)||(inst==`SUBWF)) ? 1'b1 : 1'b0;
	assign wr_zf=((inst==`ADDWF)||(inst==`ANDWF)||(inst==`CLRF)||(inst==`CLRW)||(inst==`COMF)||(inst==`DECF)||(inst==`INCF)||(inst==`IORWF)||(inst==`MOVF)||
			(inst==`SUBWF)||(inst==`XORWF)||(inst==`ANDLW)||(inst==`IORLW)||(inst==`XORLW)) ? 1'b1 : 1'b0;
	assign cmd_skip=((((inst==`DECFSZ)||(inst==`INCFSZ)||(inst==`BTFSC))&&(zf))||((inst==`BTFSS)&&(!zf))) ? 1'b1 :1'b0;
endmodule
