`include "Instruction.v"

module mCmdDecord(cmd,inst);
	input [13:0] cmd;
	output [5:0] inst;
	assign inst=
			//========= File Register Instructions =====================
			(cmd[13:8]==6'b00_0111) 	? `ADDWF :
			(cmd[13:8]==6'b00_0101)   	? `ANDWF :
			(cmd[13:7]==7'b00_0001_1) 	? `CLRF :
			(cmd[13:7]==7'b00_0001_0) 	? `CLRW :
			(cmd[13:8]==6'b00_1001)  	? `COMF :
			(cmd[13:8]==6'b00_0011)  	? `DECF :
			(cmd[13:8]==6'b00_1011)  	? `DECFSZ:
			(cmd[13:8]==6'b00_1010)  	? `INCF :
			(cmd[13:8]==6'b00_1111)  	? `INCFSZ :
			(cmd[13:8]==6'b00_0100)  	? `IORWF :
			(cmd[13:8]==6'b00_1000)  	? `MOVF :
			(cmd[12:7]==7'b00_0000_1) 	? `MOVWF :
			(cmd[13:8]==6'b00_1101)  	? `RLF :
			(cmd[13:8]==6'b00_1100)  	? `RRF :
			(cmd[13:8]==6'b00_0010)  	? `SUBWF :
			(cmd[13:8]==6'b00_1110)  	? `SWAPF :
			(cmd[13:8]==6'b00_0110)  	? `XORWF :
			//========= Bit Operation Instructions =====================
			(cmd[13:10]==4'b01_00)  	? `BCF :
			(cmd[13:10]==4'b01_01)		? `BSF :
			(cmd[13:10]==4'b01_10)		? `BTFSC :
			(cmd[13:10]==4'b01_11)		? `BTFSS :
			//========= Literal and Control Instructions ===============
			(cmd[13:9]==5'b11_111)		? `ADDLW :
			(cmd[13:10]==6'b11_1001)	? `ANDLW :
			(cmd[13:11]==3'b10_0)		? `CALL :
			(cmd==14'b00_0000_0110_0100) ? `CLRWDT :
			(cmd[13:11]==3'b10_1)		? `GOTO :
			(cmd[13:8]==6'b11_1000)		? `IORLW :
			(cmd[13:10]==4'b11_00)		? `MOVLW :
			(cmd==14'b00_0000_0000_1001) ? `RETFIE :
			(cmd[13:10]==4'b11_01)		? `RETLW :
			(cmd==14'b00_0000_0000_1000) ? `RETURN :
			(cmd==14'b00_0000_0110_0011) ? `SLEEP :
			(cmd[13:9]==5'b11_110)		? `SUBLW :
			(cmd[13:8]==6'b11_1010)		? `XORLW :
			`NOP;
endmodule

module mStack(reset,clk,Q,write,read,datin,datout);
	input reset,clk,write,read;
	input Q;
	input [12:0] datin;
	output [12:0] datout;
	reg [12:0] rstack[7:0];
	reg [2:0] sp;
	wire [2:0] rsp;

	assign rsp=sp-1'b1;
	assign datout=rstack[rsp];
	
	//Stack Pointer
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			sp<=3'h0;
		end else begin
			if(Q) begin
				if(read)begin
					sp<=sp-1'b1;
                end	else if(write) begin
					sp<=sp+1'b1;
				end
			end
		end
	end
	//Write Stack
	always@(posedge clk) begin
		if(Q & write) begin
			rstack[sp]<=datin;
		end
	end
endmodule

//Program Counter
module mPCounter(reset,clk,Q,inst,cmd,adr,wr_pc,wr_dat,intr,cmd_flash);
	input reset,clk;
	input  Q;			//Start State
	input [5:0] inst;	//Instruction Code
	input [13:0] cmd;	//Command Code(data)
	output [12:0] adr;	//Memory Address
	input wr_pc;		//Write OP to PC
	input [12:0] wr_dat;// Program Counter Write Data
	input intr;			//Hardwere Interrupt
	input cmd_flash;
	wire call,ret;
	reg [12:0] pc_reg;	//Program Counter
	reg [12:0] last_pc;	//Last Program Counter(for ISR)
	wire [12:0] sdout;	//Stack Data Output
	wire [12:0] stk_dat;
	reg dcmd_flash;
	
	assign call=((inst==`CALL)||(intr)) ? 1'b1 : 1'b0;
	assign ret=((inst==`RETLW)||(inst==`RETURN)||(inst==`RETFIE)) ? 1'b1 : 1'b0;
	assign adr=pc_reg;
	
	assign stk_dat=(intr & ~dcmd_flash) ? last_pc : pc_reg;
	
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			dcmd_flash<=1'b0;
		end else if(Q) begin
			dcmd_flash<=cmd_flash;
		end
	end
	
	//Program Counter
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			pc_reg<=13'h0;
		end 
		else if(Q) begin
			if(intr) begin
				pc_reg<=13'h004;
			end else if((inst==`GOTO)||(inst==`CALL)) begin
				pc_reg<={wr_dat[12:11],cmd[10:0]};
			end else if(ret) begin
				pc_reg<=sdout;
			end else if(wr_pc) begin
				pc_reg<=wr_dat;
			end else begin
				pc_reg<=pc_reg+1'b1;
			end
		end
	end
	//Last_PC
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			last_pc<=13'h0;
		end 
		else if(Q) begin
			last_pc<=pc_reg;
		end
	end
	
	//Stack
	mStack stk(reset,clk,Q,call,ret,stk_dat,sdout);
endmodule

//CommandProcesser
module mCmdProc(cmd,cmd_flash,inst);
	input [13:0] cmd;
	output cmd_flash;
	output [5:0] inst;
	wire [5:0] winst;
	assign inst=winst;
	assign cmd_flash=((winst==`GOTO)||(winst==`CALL)||(winst==`RETLW)||(inst==`RETURN)||(inst==`RETFIE)) ? 1'b1 : 1'b0;
	mCmdDecord CmdDec(cmd,winst);
endmodule
