
module mF84Emu(EXTCLK,LED,KEY,PORTA,PORTB,DEB);
	input EXTCLK;
	output	[7:0]	LED;
	input	[3:0]	KEY	;
	inout [7:0] PORTA;
	inout [7:0] PORTB;
	output [15:0] DEB;
	wire Q;
	wire [15:0] cmd;
	wire [12:0] adr;
	wire fosc,reset,cmd_freq,cmd_flash;
	wire [7:0] wdeb;
	wire [5:0] inst;
	wire wr_pc;
	wire [12:0] pc_in;
	wire wr_tmr;
	wire [7:0] tim_out;
	wire [7:0] opt;
	wire [7:0] db_out;
	wire wr_wreg;
	wire [7:0] wreg;
	wire [7:0] db_in;
	wire cin,cout,dc,zf,wr_fr,wr_cf,wr_dc,wr_zf,cmd_skip;
	wire [7:0] pa_in;
	wire [7:0] pa_out;
	wire [7:0] ta_out;
	wire [7:0] pb_in;
	wire [7:0] pb_out;
	wire [7:0] tb_out;
    wire [7:0] intcon_out;
	wire iosc,tovf,sel_intcon,iret,intr;
	wire Qin,Qout,ireset;
	assign cmd_flash=cmd_freq | cmd_skip | wr_pc | intr | iret;

	//Power On Reset ==============================================
	mReset Rst(ireset,iosc);
	assign reset=ireset;
	//assign reset=~KEY[0];
	assign fosc=EXTCLK;
	//assign fosc=iosc;
	//Timing Generator ============================================
	mTmgGen tmgen(reset,fosc,Qout);
	///* ============================================================ 
	//Step Debug
	//mStep step(reset,fosc,KEY[3],Qout,Qin);
	assign Q=Qout;	//Qin;
	//=============================================================== */
	//Program Memory Module =======================================
	mProgMem pmem(fosc,reset,adr,cmd,Q,cmd_flash);
	//Command Proceccer ===========================================
	mCmdProc Cproc(cmd[13:0],cmd_freq,inst);
	//Program Counter =============================================
	mPCounter PC(reset,fosc,Q,inst,cmd[13:0],adr,wr_pc,pc_in,intr,cmd_flash);
	//Timer 0 =====================================================
	mTimer0 tmr0(reset,fosc,Q,opt[3:0],wr_tmr,db_out,tim_out,tovf);
	//W-Reg =======================================================
	mWreg w_reg(reset,fosc,Q,wr_wreg,db_out,wreg);
	//ALU Module ==================================================
	mALU alu(inst,cmd[13:0],wreg,db_in,cin,db_out,cout,dc,zf,wr_fr,wr_wreg,wr_cf,wr_dc,wr_zf,cmd_skip);
	//File Register ==============================================
	mRegFile regfile(reset,fosc,Q,cmd[6:0],db_out,db_in,cout,cin,dc,zf,wr_cf,wr_dc,wr_zf,wr_fr,pa_in,pa_out,ta_out,pb_in,pb_out,tb_out,adr,pc_in,wr_pc,tim_out,wr_tmr,opt,sel_intcon,intcon_out);	
    //Interrupt Control===========================================
    assign iret=(inst==`RETFIE) ? 1'b1 : 1'b0;
    mIntrc intrc(reset,fosc,Q,pb_in,opt[6],tovf,iret,sel_intcon,wr_fr,db_out,intcon_out,intr);
	//GPIO Setting  ==============================================
    //PORTA
	assign PORTA[0]=(ta_out[0]) ? 1'bz : pa_out[0];
	assign PORTA[1]=(ta_out[1]) ? 1'bz : pa_out[1];
	assign PORTA[2]=(ta_out[2]) ? 1'bz : pa_out[2];
	assign PORTA[3]=(ta_out[3]) ? 1'bz : pa_out[3];
	assign PORTA[4]=(ta_out[4]) ? 1'bz : pa_out[4];
	assign PORTA[5]=(ta_out[5]) ? 1'bz : pa_out[5];
	assign PORTA[6]=(ta_out[6]) ? 1'bz : pa_out[6];
	assign PORTA[7]=(ta_out[7]) ? 1'bz : pa_out[7];
    //PORTB
	assign PORTB[0]=(tb_out[0]) ? 1'bz : pb_out[0];
	assign PORTB[1]=(tb_out[1]) ? 1'bz : pb_out[1];
	assign PORTB[2]=(tb_out[2]) ? 1'bz : pb_out[2];
	assign PORTB[3]=(tb_out[3]) ? 1'bz : pb_out[3];
	assign PORTB[4]=(tb_out[4]) ? 1'bz : pb_out[4];
	assign PORTB[5]=(tb_out[5]) ? 1'bz : pb_out[5];
	assign PORTB[6]=(tb_out[6]) ? 1'bz : pb_out[6];
	assign PORTB[7]=(tb_out[7]) ? 1'bz : pb_out[7];

	//for Standerd I/O  ==========================================
	//assign LED=~pb_out; 
	//assign pa_in={4'h0,KEY};
	
	//for LED debug==============================================
	/*
	assign LED=(KEY==4'he) ? ~cmd[7:0] : 
		(KEY==4'hd) ? ~cmd[15:8] :
		(KEY==4'hb) ? ~db_out :
		(KEY==4'ha) ? ~db_in :
		(KEY==4'hc) ? ~wreg :
		(KEY==4'h8) ? ~{cout,zf,inst} :
		(KEY==4'h9) ? ~tim_out :~adr[7:0];
	*/
	assign pa_in=PORTA;
	assign pb_in=PORTB;
	assign LED=~pb_out;	//adr[7:0];
	assign DEB={intr,tovf,intcon_out[7],iret,adr[11:0]};
	//==========================================================
	
endmodule
