//Register Files

//Status Register

module mStatus(reset,clk,Q,sel,wr_zf,wr_dc,wr_cf,wr_fr,zf,dc,cf,datin,datout);
	input reset,clk;
	input Q,sel;
	input wr_zf,wr_dc,wr_cf,wr_fr,zf,dc,cf;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] streg;
	
	assign datout=streg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			streg<=8'h18;
		end else if(Q) begin
			if(sel & wr_fr) begin
				streg<=datin;
			end else begin
				if(wr_zf)
					streg[2]<=zf;
				if(wr_dc)
					streg[1]<=dc;
				if(wr_cf)
					streg[0]<=cf;
			end
		end
	end
endmodule

//Option Register
module mOption(reset,clk,Q,sel,wr_opt,datin,datout);
	input reset,clk;
	input Q,sel;
	input wr_opt;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] optreg;
	assign datout=optreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			optreg<=8'hff;
		end else if(Q) begin
			if(sel & wr_opt) begin
				optreg<=datin;
			end
		end
	end
endmodule


//FSR Register
module mFsr(reset,clk,Q,sel,wr_fsr,datin,datout);
	input reset,clk;
	input Q,sel;
	input wr_fsr;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] fsrreg;
	assign datout=fsrreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			fsrreg<=8'hff;
		end else if(Q) begin
			if(sel & wr_fsr) begin
				fsrreg<=datin;
			end
		end
	end
endmodule


//General Reg
module mGpReg(reset,clk,Q,sel,wr_fr,datin,datout);
	input reset,clk;
	input Q,sel;
	input wr_fr;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] gpreg;
	assign datout=gpreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			gpreg<=8'h00;
		end else if(Q & sel & wr_fr) begin
				gpreg<=datin;
		end
	end
endmodule

module mRegFile(
	reset,clk,Q,	//Reset,fosc,Q state
	fadr,			//File Reg Address
	//ALU Interface
	datin,
	datout,
	cin,cout,dc,zf,wr_cf,wr_dc,wr_zf,wr_fr,
	//GPIO
	pa_in,pa_out,ta_out,pb_in,pb_out,tb_out,
	//Program Counter
	pc_in,pc_out,wr_pc,
	//Timer
	timer_in,wr_tmr,
	//OPtion Reg 
	opt_out,
	//Intcon
	sel_intcon,
	intcon_out
	);
	input reset,clk;
	input Q;
	input [6:0] fadr;
	input [7:0] datin;
	output [7:0] datout;
	input cin;
	output cout;
	input dc,zf,wr_cf,wr_dc,wr_zf,wr_fr; 
	//PORTA
	input [7:0] pa_in;		//input from GPIO pin
	output [7:0] pa_out;	//Output to GPIO
	output [7:0] ta_out;	//TRIS Out
	//PORTB
	input [7:0] pb_in;		//input from GPIO pin
	output [7:0] pb_out;	//Output to GPIO
	output [7:0] tb_out;	//TRIS Out
	input [12:0] pc_in;	//Program Counter
	output [12:0] pc_out;
	output wr_pc;
	input [7:0] timer_in;
	output wr_tmr;
	output [7:0] opt_out;
	output sel_intcon;	//INTCON REG
	input [7:0] intcon_out;
	wire [8:0] iadr;	//internal Address
	wire [7:0] fsr_out;	//FSR Reg
	wire [7:0] st_out;	//Status Reg
	wire [7:0] pch_out;
	//EEPROM Control
	wire [7:0] eedata_out;
	wire [7:0] eeadr_out;
	wire [7:0] eecon1_out;
	wire [7:0] eecon2_out;
	reg [7:0] ram[12:79]; //RAM
	//Register Select Signals
	wire sel_tmr,sel_pcl,sel_st,sel_fsr,sel_pa,sel_pb,sel_eedat,sel_eeadr,sel_pch,sel_opt,sel_eecon1,sel_eecon2,sel_ram;
	
	assign cout=st_out[0];	
	assign pc_out={pch_out[4:0],datin};
	assign iadr=(fadr==7'h00) ? {st_out[7],fsr_out} :{st_out[6:5],fadr};
	//Decorder
	assign sel_tmr=(iadr==9'h1) ? 1'b1 : 1'b0;
	assign sel_pcl=(iadr[6:0]==7'h2) ? 1'b1 : 1'b0;
	assign sel_st=(iadr[6:0]==7'h3) ? 1'b1 : 1'b0;
	assign sel_fsr=(iadr[6:0]==7'h4) ? 1'b1 : 1'b0;
	assign sel_pa=(iadr==9'h5) ? 1'b1 : 1'b0;
	assign sel_pb=(iadr==9'h6) ? 1'b1 : 1'b0;
	assign sel_eedat=(iadr==9'h08) ? 1'b1 : 1'b0;
	assign sel_eeadr=(iadr==9'h09) ? 1'b1 : 1'b0;
	assign sel_pch=(iadr[6:0]==7'h0a) ? 1'b1 : 1'b0;
	assign sel_intcon=(iadr[6:0]==7'h0b) ? 1'b1 : 1'b0;
	assign sel_opt=(iadr==9'h81) ? 1'b1 : 1'b0;
	assign sel_ta=(iadr==9'h85) ? 1'b1 : 1'b0;
	assign sel_tb=(iadr==9'h86) ? 1'b1 : 1'b0;
	assign sel_eecon1=(iadr==9'h88) ? 1'b1 : 1'b0;
	assign sel_eecon2=(iadr==9'h89) ? 1'b1 : 1'b0;
	assign sel_ram=((iadr[6:0]>=7'h0c)||(iadr[6:0]<=7'h4f)) ? 1'b1 : 1'b0;

	assign datout=(sel_tmr) ? timer_in :
		(sel_pcl) ? pc_in[7:0] :
		(sel_st) ? st_out :
		(sel_fsr) ? fsr_out :
		(sel_pa) ? pa_in :
		(sel_pb) ? pb_in :
		(sel_pch) ? pch_out :
		(sel_intcon) ? intcon_out :
		(sel_opt) ? opt_out :
		(sel_ta) ? ta_out :
		(sel_tb) ? tb_out :
		(sel_eedat) ? eedata_out :
		(sel_eeadr) ? eeadr_out :
		(sel_eecon1) ? eecon1_out :
		(sel_eecon2) ? eecon2_out :
		(sel_ram) ? ram[iadr] : 8'h0;
	//Write signals
	assign wr_tmr=(sel_tmr) ? wr_fr : 1'b0;
	assign wr_pc=(sel_pcl) ? wr_fr : 1'b0;
	
	always@(posedge clk) begin
		if(Q) begin
			if(sel_ram & wr_fr) begin
				ram[iadr]<=datin;
			end
		end
	end
	mStatus streg(reset,clk,Q,sel_st,wr_zf,wr_dc,wr_cf,wr_fr,zf,dc,cin,datin,st_out);
	mOption optreg(reset,clk,Q,sel_opt,wr_fr,datin,opt_out);
	mFsr fsrreg(reset,clk,Q,sel_fsr,wr_fr,datin,fsr_out);
	//PORTA
	mGpReg porta(reset,clk,Q,sel_pa,wr_fr,datin,pa_out);
	mGpReg trisa(reset,clk,Q,sel_ta,wr_fr,datin,ta_out);
	//PORTB
	mGpReg portb(reset,clk,Q,sel_pb,wr_fr,datin,pb_out);
	mGpReg trisb(reset,clk,Q,sel_tb,wr_fr,datin,tb_out);
	//PCL
	mGpReg pch(reset,clk,Q,sel_pch,wr_fr,datin,pch_out);
	//EEPROM Control
	mGpReg eedata(reset,clk,Q,sel_eedat,wr_fr,datin,eedata_out);
	mGpReg eeadr(reset,clk,Q,sel_eeadr,wr_fr,datin,eeadr_out);
	mGpReg eecon1(reset,clk,Q,sel_eecon1,wr_fr,datin,eecon1_out);
	mGpReg eecon2(reset,clk,Q,sel_eecon2,wr_fr,datin,eecon2_out);
	
endmodule
